#include <NewPing.h>
#include <Servo.h>

#define TRIG_PIN A0
#define ECHO_PIN A1
#define MAX_DISTANCE 400

#define S0 A2
#define S1 A3
#define S2 A4
#define S3 A5
#define sensorOut 8

#define IN1 4
#define IN2 5
#define IN3 6
#define IN4 7

#define PUMP_PIN 9

int blueCheck = 0;


NewPing sonar(TRIG_PIN, ECHO_PIN, MAX_DISTANCE);

Servo eyeServo;
Servo paintServo;

int distance = 100;
int pos = 0;

void setup() {
  Serial.begin(9600); // Starting Serial Terminal

  // set up color sensors
  pinMode(S0, OUTPUT);
  pinMode(S1, OUTPUT);
  pinMode(S2, OUTPUT);
  pinMode(S3, OUTPUT);
  pinMode(sensorOut, INPUT);
  digitalWrite(S0, HIGH);
  digitalWrite(S1, LOW);



  //set up motors
  pinMode(IN4, OUTPUT);  //left motors forward
  pinMode(IN3, OUTPUT);  //left motors reverse
  pinMode(IN2, OUTPUT);  //right motors forward
  pinMode(IN1, OUTPUT);  //right motors reverse

  //set up pumping motor
  pinMode(PUMP_PIN, OUTPUT);

  // set up servo
  eyeServo.attach(3);
  eyeServo.write(90);
  delay(2000);

  paintServo.attach(10);
  paintServo.write(90);
  delay(2000);

  distance = readPing();
  delay(60);

  
}

void loop() {
  Serial.print(distance);
  Serial.println();
  int distanceR = 0;
  int distanceL =  0;

  if (distance <= 20)
  {
    moveStop();
    delay(1000);
    blueCheck = colorRead();
    Serial.print(blueCheck);
    if (blueCheck == 1) {
      pump();
    }
    blueCheck = 0;
    delay(500);
    moveBackward();
    delay(400);
    moveStop();
    delay(100);
    distanceR = lookRight();
    delay(50);
    distanceL = lookLeft();
    delay(50);
    Serial.print(distanceR);
    Serial.print(distance
    );

    if (distanceR >= distanceL)
    {
        turnLeft();
        moveStop();
        delay(100);
        
    } else if (distanceR < distanceL)
    {
        turnRight();
        moveStop();
        delay(100);
      }
  } else moveForward();

  distance = readPing();
  delay(60);
}

int lookLeft()
{
  for (pos = 90; pos <= 180; pos += 1) { // goes from 90 degrees to 180 degrees
    // in steps of 1 degree
    eyeServo.write(pos);              // tell servo to go to position in variable 'pos'
    delay(5);
  }
  int distance = readPing();
  delay(60);

  for (pos = 180; pos >= 90; pos -= 1) { // goes from 180 degrees to 90 degrees
    // in steps of 1 degree
    eyeServo.write(pos);              // tell servo to go to position in variable 'pos'
    delay(5);
  }
  return distance; // return distance on the left side
}

int lookRight()
{
  for (pos = 90; pos >= 0; pos -= 1) { // goes from 90 degrees to 0 degree
    // in steps of 1 degree
    eyeServo.write(pos);              // tell servo to go to position in variable 'pos'
    delay(5);
  }
  int distance = readPing();
  delay(60);

  for (pos = 0; pos <= 90; pos += 1) { // goes from 0 degree to 90 degrees
    // in steps of 1 degree
    eyeServo.write(pos);              // tell servo to go to position in variable 'pos'
    delay(5);
  }
  return distance; // return distance on the right side
}

int readPing() {
  int cm = sonar.ping_cm();
  if (cm == 0) // if the measured distance exceeds the measuring limit of the ultrasonic sensor, by default the sonar.ping_cm() function will return a value of 0
  { // Due to the distance of 0 the robot will keep avoiding even when there is nothing in front of it
    cm = MAX_DISTANCE; // To fix this, we set the distance to the pre-defined MAX_DISTANCE of 400 cm
  }
  return cm;
}

void moveStop() {
  digitalWrite(IN4, LOW);
  digitalWrite(IN3, LOW);
  digitalWrite(IN2, LOW);
  digitalWrite(IN1, LOW);
}

void moveForward() {
  digitalWrite(IN4, HIGH);
  digitalWrite(IN2, HIGH);
}

void moveBackward() {
  digitalWrite(IN3, HIGH);
  digitalWrite(IN1, HIGH);
}

void turnRight() {
  digitalWrite(IN1, HIGH);
  delay(250);
  digitalWrite(IN4, HIGH);
  digitalWrite(IN2, HIGH);
}

void turnLeft() {
  digitalWrite(IN3, HIGH);
  delay(250);
  digitalWrite(IN4, HIGH);
  digitalWrite(IN2, HIGH);
}

int colorRead() {

  int redFrequency = 0;
  int greenFrequency = 0;
  int blueFrequency =  0;

  int check;
  // Setting RED (R) filtered photodiodes to be read
  digitalWrite(S2, LOW);
  digitalWrite(S3, LOW);

  // Reading the output frequency
  redFrequency = pulseIn(sensorOut, LOW);

  // Printing the RED (R) value
  Serial.print("R = ");
  Serial.print(redFrequency);
  delay(100);

  // Setting GREEN (G) filtered photodiodes to be read
  digitalWrite(S2, HIGH);
  digitalWrite(S3, HIGH);

  // Reading the output frequency
  greenFrequency = pulseIn(sensorOut, LOW);

  // Printing the GREEN (G) value
  Serial.print(" G = ");
  Serial.print(greenFrequency);
  delay(100);

  // Setting BLUE (B) filtered photodiodes to be read
  digitalWrite(S2, LOW);
  digitalWrite(S3, HIGH);

  // Reading the output frequency
  blueFrequency = pulseIn(sensorOut, LOW);

  // Printing the BLUE (B) value
  Serial.print(" B = ");
  Serial.println(blueFrequency);
  delay(100);

  
  if (redFrequency < greenFrequency && redFrequency < blueFrequency) {
    Serial.println(" - RED detected!");
    delay(500);
    check = 1;
  }
  if (greenFrequency < redFrequency && greenFrequency < blueFrequency) {
    Serial.println(" - GREEN detected!");
    delay(500);
    check = 1;
  }
  if (blueFrequency < redFrequency && blueFrequency < greenFrequency) {
    Serial.println(" - BLUE detected!");
    delay(500);
    check = 0;
  }

return check;
}

int paint()
{
  for (pos = 90; pos <= 180; pos += 1) { // goes from 90 degrees to 180 degrees
    // in steps of 1 degree
    paintServo.write(pos);              // tell servo to go to position in variable 'pos'
    delay(5);
  }
  for (pos = 180; pos >= 90; pos -= 1) { // goes from 180 degrees to 90 degrees
    // in steps of 1 degree
    paintServo.write(pos);              // tell servo to go to position in variable 'pos'
    delay(5);
  }
}

void pump()
{
  digitalWrite(PUMP_PIN,HIGH);
  delay(2500);
  digitalWrite(PUMP_PIN,LOW); 
}
